clear;clc;
addpath('..')

n1 = 100; n2 = 100; n3 = 100; nn = [n1 n2 n3];
n = n1+n2+n3;
N = 2*(n1 + n2) + n3; M = 100;
sM = linspace(0, 1, M);

beta = [beta1(sM); beta2(sM); beta3(sM)];

eta11 = eta1(sM); eta21 = eta2(sM);
eta31 = eta3(sM); eta41 = eta4(sM);

lambda = [0.05*[1 0.5] 0.05*[1 0.5]];

SigmaA = lambda(1)*(eta11')*eta11 + lambda(2)*(eta21')*eta21;
SigmaC = lambda(3)*(eta31')*eta31 + lambda(4)*(eta41')*eta41;

SigmaE = 0.0001;

rng(1234567);

[y, X, famid, zygosity, eta] = ...
    FSEM_gen1data(beta, SigmaA, SigmaC, SigmaE, nn);
[MZtp1,~,DZtp1,~,MDZti] = FSEM_index(famid,zygosity);
proportion = 0.8;
NN = 1:N;
nn1 = NN(MZtp1); nn2 = NN(DZtp1); nn3 = NN(MDZti);
nn1train = randsample(nn1, floor(n1*proportion)); 
nn2train = randsample(nn2, floor(n2*proportion)); 
nn3train = randsample(nn3, floor(n3*proportion));

NNtrain = sort([nn1train,nn1train+1,nn2train,nn2train+1,nn3train]);
NNtest = setdiff(NN, NNtrain);

[paramsA, ~] = FSEM_mle(y(NNtrain,:), X(NNtrain,:), famid(NNtrain), zygosity(NNtrain));
beta_est = paramsA(4:end,:); 
R = y(NNtrain,:) - X(NNtrain,:)*beta_est;
ob = FSEM_cov_ob(R, famid(NNtrain), zygosity(NNtrain), sM);
[SigmaA_est, SigmaC_est] = ...
    FSEM_cov(R, famid(NNtrain), zygosity(NNtrain), sM, ob);

% k-l expansion
[P, Da] = eig(SigmaA_est);
[Q, Dc] = eig(SigmaC_est);
Da = real(Da); Dc = real(Dc);
indxa = diag(Da) > 1e-3;
indxc = diag(Dc) > 1e-3;
P = P(:,indxa); Q = Q(:,indxc);
MM = [P, Q];

y_test = y(NNtest,:);

indx_train = randsample(M,70);
kk = 1;
ykk = y_test(kk,:)';
coef_t = FSEM_predict(ykk, MM);
ykk_p = MM*coef_t;

nrep = 200;
coef1 = zeros(length(coef_t),nrep);
coef2 = zeros(length(coef_t),nrep);

for kk = 1:nrep
    indx_train1 = randsample(M, 80);
    indx_train2 = 1:80;
    ytmp = ykk_p + randn(length(ykk_p),1)*0.2;
    coef1(:,kk) = FSEM_predict(ytmp(indx_train1), MM(indx_train1,:));
    coef2(:,kk) = FSEM_predict(ytmp(indx_train2), MM(indx_train2,:));
end
tmp1 = coef1-repmat(coef_t,1,nrep);
tmp2 = coef2-repmat(coef_t,1,nrep);
tmp1 = tmp1.^2; tmp2 = tmp2.^2;
[mean(tmp1(:)), mean(tmp2(:))]
[std(tmp1(:)), std(tmp2(:))]

figure;
plot(coef_t,'r'); hold on; plot(coef,'k');
