clear; clc;

addpath('..')
addpath('../src')

load('demo.mat');

%% preprocessing

[MZtp1,~,DZtp1,~,MDZti] = FSEM_index(famid,zygosity);
n = sum(MZtp1) + sum(DZtp1) + sum(MDZti);

age_MRI = (age_MRI-mean(age_MRI))/std(age_MRI);
gender = (gender-mean(gender))/std(gender);
X = [ones(length(age_MRI),1),gender,age_MRI];

FA = csvread('FA_GENU.csv', 1, 1)';
FA(end,:) = [];
FA(:, any(isnan(FA), 1)) = []; % remove all columns with NaN

MD = csvread('MD_GENU.csv', 1, 1)';
MD(end,:) = [];
MD(:, any(isnan(MD), 1)) = []; % remove all columns with NaN

meanFA = mean(FA(:));
meanMD = mean(MD(:));
maxMean = max([meanFA, meanMD]);
FA = FA*(maxMean/meanFA);
MD = MD*(maxMean/meanMD);

sM = linspace(0, 1, size(FA,2)); 
h = 0.1*n^(-1/3);

%% FA data
% mle estimation and test
[FA_paramsA, FA_paramsN] = FSEM_mle(FA, X, famid, zygosity);
[FA_LRT_vec, FA_p_vec] = FSEM_lrt(FA_paramsA, FA_paramsN, FA, X, famid, zygosity);
% wmle estimation and test
[FA_betaA, FA_log_WsigmaA, FA_log_WsigmaN] = FSEM_wmle(FA, X, famid, zygosity, sM, h);
FA_R = FA - X*FA_betaA;
[FA_WLRT_vec, FA_Wp_vec] = FSEM_wlrt(FA_log_WsigmaN, FA_R, famid, zygosity, sM, h);
% global test
FA_p = FSEM_global_test(FA_R, famid, zygosity);
% covariance structure estimation
FA_ob = FSEM_cov_ob(FA_R, famid, zygosity, sM);
[FA_SigmaA_est, FA_SigmaC_est, FA_SigmaEG_est] = FSEM_cov(FA_R, famid, zygosity, sM, FA_ob);

%% MD data 
% mle estimation and test
[MD_paramsA, MD_paramsN] = FSEM_mle(MD, X, famid, zygosity);
[MD_LRT_vec, MD_p_vec] = FSEM_lrt(MD_paramsA, MD_paramsN, MD, X, famid, zygosity);
% wmle estimation and test
[MD_betaA, MD_log_WsigmaA, MD_log_WsigmaN] = FSEM_wmle(MD, X, famid, zygosity, sM, h);
MD_R = MD - X*MD_betaA;

[MD_WLRT_vec, MD_Wp_vec] = FSEM_wlrt(MD_log_WsigmaN, MD_R, famid, zygosity, sM, h);
% global test
MD_p = FSEM_global_test(MD_R, famid, zygosity);
% covariance structure estimation
MD_ob = FSEM_cov_ob(MD_R, famid, zygosity, sM);
[MD_SigmaA_est, MD_SigmaC_est, MD_SigmaEG_est] = ...
    FSEM_cov(MD_R, famid, zygosity, sM, MD_ob);

%% confidence bands for beta
M = size(MD,2);
FA_paramsA_se = zeros(6,M); 
MD_paramsA_se = zeros(6,M);

for kk = 1:M
    tmp = hessian(@(params) FSEM_lla(params,FA(:,kk),X,famid,zygosity), ...
        FA_paramsA(:,kk));
    tmp = diag(inv(tmp)); 
    FA_paramsA_se(:,kk) = sqrt(2*tmp).*[exp(FA_paramsA(1:3,kk));ones(3,1)];
    
    tmp = hessian(@(params) FSEM_lla(params,MD(:,kk),X,famid,zygosity), ...
        MD_paramsA(:,kk));
    tmp = diag(inv(tmp)); 
    MD_paramsA_se(:,kk) = sqrt(2*tmp).*[exp(MD_paramsA(1:3,kk));ones(3,1)];
end

save real_data.mat