clear; clc;

addpath('..')

load('demo.mat');

% preprocessing

[MZtp1,~,DZtp1,~,MDZti] = FSEM_index(famid,zygosity);
n = sum(MZtp1) + sum(DZtp1) + sum(MDZti);

age_MRI = (age_MRI-mean(age_MRI))/std(age_MRI);
gender = (gender-mean(gender))/std(gender);
X = [ones(length(age_MRI),1),gender,age_MRI];

y = csvread('FA_GENU.csv', 1, 1)';
y(end,:) = [];
y(:, any(isnan(y), 1)) = []; % remove all columns with NaN

sM = linspace(0, 1, size(y,2));

proportion = 0.8;
N = 2*n - sum(MDZti);
NN = 1:N;
n1 = sum(MZtp1); n2 = sum(DZtp1); n3 = sum(MDZti);
nn1 = NN(MZtp1); nn2 = NN(DZtp1); nn3 = NN(MDZti);

error1 = zeros(10,1);
error2 = zeros(10,1);

for ii = 1:10
    
    rng(1234567*ii)

    nn1train = randsample(nn1, floor(n1*proportion));
    nn2train = randsample(nn2, floor(n2*proportion));
    nn3train = randsample(nn3, floor(n3*proportion));

    NNtrain = sort([nn1train,nn1train+1,nn2train,nn2train+1,nn3train]);
    NNtest = sort(setdiff(NN, NNtrain));

    [paramsA, ~] = FSEM_mle(y(NNtrain,:), X(NNtrain,:), famid(NNtrain), zygosity(NNtrain));
    beta = paramsA(4:end,:);
    R_train = y(NNtrain,:) - X(NNtrain,:)*beta;
    ob = FSEM_cov_ob(R_train, famid(NNtrain), zygosity(NNtrain), sM);
    [SigmaA, SigmaC, SigmaE] = ...
        FSEM_cov(R_train, famid(NNtrain), zygosity(NNtrain), sM, ob);

    [P, Da] = eig(SigmaA);
    [Q, Dc] = eig(SigmaC);
    [R, De] = eig(SigmaE);
    indxa = diag(Da) > 1e-4;
    indxc = diag(Dc) > 1e-4;
    indxe = diag(De) > 1e-4;
    P = P(:,indxa); Q = Q(:,indxc); R = R(:,indxe);
    M = [P, Q, R];

    N_G = length(sM);

    R_test = y(NNtest,:)-X(NNtest,:)*beta;
    error2t = R_test.^2;
    error2(ii) = mean(sum(error2t,2));

    coef = (M'*M)\(M'*R_test');
    R_test_pred = (M*coef)';

    y_test = y(NNtest,:);
    y_test_pred = X(NNtest,:)*beta + R_test_pred;

    error1t = (R_test-R_test_pred).^2;
    error1(ii) = mean(sum(error1t,2));
end



figure; 
subplot(2,2,1);
plot(y_test(1,:),'r'); hold on; plot(y_test_pred(1,:),'g');
legend('Observed','Estimated')
title('Observed and Estimated Functional Response','FontSize',14); 
xlabel('arclength-(a)','FontSize',10);
subplot(2,2,2);
plot(y_test(2,:),'r'); hold on; plot(y_test_pred(2,:),'g');
legend('Observed','Estimated')
title('Observed and Estimated Functional Response','FontSize',14); 
xlabel('arclength-(b)','FontSize',10);
subplot(2,2,3);
plot(y_test(3,:),'r'); hold on; plot(y_test_pred(3,:),'g');
legend('Observed','Estimated')
title('Observed and Estimated Functional Response','FontSize',14); 
xlabel('arclength-(c)','FontSize',10);
subplot(2,2,4);
plot(y_test(4,:),'r'); hold on; plot(y_test_pred(4,:),'g');
legend('Observed','Estimated')
title('Observed and Estimated Functional Response','FontSize',14); 
xlabel('arclength-(d)','FontSize',10);

figure; 
subplot(2,2,1);
plot(R_test(1,:),'r'); hold on; plot(R_test_pred(1,:),'g');
legend('Residuals','Estimated')
title('Estimated Residuals via FPCA','FontSize',14); 
xlabel('arclength-(a)','FontSize',10);
subplot(2,2,2);
plot(R_test(2,:),'r'); hold on; plot(R_test_pred(2,:),'g');
legend('Residuals','Estimated')
title('Estimated Residuals via FPCA','FontSize',14); 
xlabel('arclength-(b)','FontSize',10);
subplot(2,2,3);
plot(R_test(3,:),'r'); hold on; plot(R_test_pred(3,:),'g');
legend('Residuals','Estimated')
title('Estimated Residuals via FPCA','FontSize',14); 
xlabel('arclength-(c)','FontSize',10);
subplot(2,2,4);
plot(R_test(4,:),'r'); hold on; plot(R_test_pred(4,:),'g');
legend('Residuals','Estimated')
title('Estimated Residuals via FPCA','FontSize',14); 
xlabel('arclength-(d)','FontSize',10);
