% FSEM_mle does estimation at all tract points
%   using maximum likelihood approach

% argin
% y: N x M vector
% X: N x p vector
% famid: N x 1 vector
% zygosity: N x 1 vector

% argout
% paramsA:   (3+p) x M matrix
% paramsN_a: (2+p) x M matrix
% paramsN_c: (2+p) x M matrix

% 08/12/2017
% Shikai Luo

function [paramsA, paramsN] = FSEM_mle(y, X, famid, zygosity)

M = size(y, 2);
p = size(X, 2);

options = optimoptions('fminunc','GradObj','off', 'Display','off','Algorithm','quasi-newton');

if nargout == 1
    paramsA = zeros(p+3, M);
    % do analysis at each tract point
    for kk = 1:M
        paramsA(:,kk) = fminunc(@(params) FSEM_lla(params, y(:,kk), X, famid, zygosity), ...
            zeros(p+3,1), options);
    end
end

if nargout == 2
    paramsA = zeros(p+3, M);
    paramsN = zeros(p+2, M);
    % do analysis at each tract point
    for kk = 1:M
        paramsA(:,kk) = fminunc(@(params) FSEM_lla(params, y(:,kk), X, famid, zygosity), ...
            zeros(p+3,1), options);
        paramsN(:,kk) = fminunc(@(params) FSEM_lln(params, y(:,kk), X, famid, zygosity), ...
            zeros(p+2,1), options);
    end
end

end