% argin
% log_WsigmaA: 3 x M matrix
% log_WsigmaN: 2 x M matrix
% R: N x M matrix
% famid: N x 1 vector
% zygosity: N x 1 vector
% sM: 1 x M vector
% h: scalar or 1 x K vector
% region: 1 x K vector
%
% argout
% WLRT1: test statistic integration case
% Wp1:  p value
% WLRT2: test statistic supremum case
% Wp2: p value.

function [WLRT_vec, Wp_vec, WLRT1, Wp1, WLRT2, Wp2] = ...
    FSEM_test(log_WsigmaA, log_WsigmaN, R, famid, zygosity, sM, ...
    h, region, type)

if nargin < 9
    type = 'both';
end

if nargin < 8
    type = 'both';
    region = sM;
end

if nargin < 7
   type = 'both';
   region = sM;
   h = 5*range(sM)/length(sM)*ones(1,length(region));   
end

K = length(region);

if length(h) == 1
    h = h*ones(1, K);
end

[MZtp1, ~, DZtp1, ~, MDti] = FSEM_index(famid, zygosity);
n1 = sum(MZtp1); n2 = sum(DZtp1); n3 = sum(MDti);
n = n1 + n2 + n3;

BB = 1000; % bootstrap size
ksi = randn(BB, n);

WLRT_vec = zeros(1, K);
Wp_vec = zeros(1, K);

T_BB = zeros(BB, K);

for kk = 1:K
    % score matrix n x 3 
    J_n = FSEM_wlla([-100;log_WsigmaN(:,kk)], R, famid, zygosity, sM, ...
        loc(kk), h(kk), 1);
    J_n = J_n/sqrt(n); 
    % information matrix
    I_n = FSEM_wlla([-100;log_WsigmaN(:,kk)], R, famid, zygosity, sM, ...
        loc(kk), h(kk), 2)/n;
    [V, D] = eig(I_n);
    D = D(1,1);
    
%     fn = FSEM_wlln(log_WsigmaN(:,kk), R, famid, zygosity, sM, ...
%         region(kk), h(kk));
%     fa = FSEM_wlla(log_WsigmaA(:,kk), R, famid, zygosity, sM, ...
%         region(kk), h(kk));
%     WLRT_vec(kk) = 2*max(0, fn-fa);

    CC = sum(J_n, 1)*V;
    CC = CC(1);
    WLRT_vec(kk) = max(0, CC^2*(CC > 0)/D);
    
    J_n = ksi*J_n; % BB x 3 matrix
    C1 = J_n*V; % BB x 3 matrix
    C1 = C1(:,1); % BB x 1 vector
    
    T_BB = max(0, C1.^2.*(C1*D > 0)/D);
    
    Wp_vec(kk) = mean(T_BB > WLRT_vec(kk));
end

if nargout > 4
    type = 'both';
end

if strcmp(type, 'integration')
    WLRT1 = mean(WLRT_vec);
    T_1 = mean(T_BB, 2);
    Wp1 = mean(T_1 > WLRT1);
elseif strcmp(type, 'supremum') 
    WLRT2 = max(WLRT_vec);
    T_2 = max(T_BB, [], 2);
    Wp2 = mean(T_2 > WLRT2);
else
    WLRT1 = mean(WLRT_vec);
    T_1 = mean(T_BB, 2);
    Wp1 = mean(T_1 > WLRT1);
    WLRT2 = max(WLRT_vec);
    T_2 = max(T_BB, [], 2);
    Wp2 = mean(T_2 > WLRT2);
end

end