% FSEM_wlrt_a does pointwise test given parameter estimates via WLRT

% argin
% log_WsigmaA: 3 x K matrix
% log_WsigmaN: 2 x K matrix
% R: N x M matrix
% famid: N x 1 vector
% zygosity: N x 1 vector
% sM: 1 x M vector
% h: scalar or 1 x K vector
% loc: 1 x K vector, locations corresponding to log_WsigmaA and log_WsigmaN

% argout
% WLRT_vec: 1 x K vector;
% Wp_vec: 1 x K vector; p values

% 08/12/2017
% Shikai Luo

function [WLRT_vec, Wp_vec] = ...
    FSEM_wlrt(log_WsigmaN, R, famid, zygosity, sM, h, loc)

if nargin < 7 && size(log_WsigmaN,2) == size(R,2)
    loc = sM;
end

if nargin < 6
    h = 5*range(sM)/length(sM)*ones(1,length(loc));
end

K = length(loc);

if length(h) == 1
    h = h*ones(1, K);
end

[MZtp1, ~, DZtp1, ~, MDti] = FSEM_index(famid, zygosity);
n1 = sum(MZtp1); n2 = sum(DZtp1); n3 = sum(MDti);
n = n1 + n2 + n3;

BB = 500; % bootstrap size (BB=1000 for simulation)
ksi = randn(BB, n);

WLRT_vec = zeros(1, K);
Wp_vec = zeros(1, K);

for kk = 1:K
    % score matrix n x 3 
    J_n = FSEM_wlla([-100;log_WsigmaN(:,kk)], R, famid, zygosity, sM, ...
        loc(kk), h(kk), 1);
    J_n = J_n/sqrt(n); 
    % information matrix
    I_n = FSEM_wlla([-100;log_WsigmaN(:,kk)], R, famid, zygosity, sM, ...
        loc(kk), h(kk), 2)/n;
    [V, D] = eig(I_n);
    D = D(1,1);
    
    CC = sum(J_n, 1)*V;
    CC = CC(1);
    WLRT_vec(kk) = max(0, CC^2*(CC > 0)/D);
    
    J_n = ksi*J_n; % BB x 3 matrix
    C1 = J_n*V; % BB x 3 matrix
    C1 = C1(:,1); % BB x 1 vector
    T_BB = max(0, C1.^2.*(C1 > 0)/D);
    
    Wp_vec(kk) = mean(T_BB > WLRT_vec(kk));
end

end